#!/bin/sh
#
# test jmerge with large files
#  grep f_msg ~/src/util/jmerge/matnr.sh | cut -c 1-78 | rtw -J | sed 's/"[IEW]/"/' | sort
#

set -o pipefail

. $HOME/bin/catalog.sh

f_msg()
{
    l_lm_msg="$1"
    l_lm_tmsg=`echo "$l_lm_msg" | cut -c 1-1`
    l_lm_now="`date '+%H:%M:%S'`"
    
    case "$l_lm_tmsg" in
	"W")
	    echo "$l_lm_now - WARN  $l_lm_msg"
	    ;;
	"S")
	    echo "$l_lm_now - SUCCESS $l_lm_msg"
	    ;;
	"I")
	    echo "$l_lm_now - INFO  $l_lm_msg"
	    ;;
	"E")
	    echo "$l_lm_now - ERROR $l_lm_msg"
	    exit 2
	    ;;
	*)
	    echo "$l_lm_now - ABORT $l_lm_msg"
	    exit 2
	    ;;
    esac

} # END: f_msg()

f_trap()
{
    f_msg "W900: Cancelled, cleaning"

    if test "$g_generating_files" = "Y"
    then
	if test -f "$g_dir_matnr/MARA.txt"
	then
	    rm -v "$g_dir_matnr/MARA.txt"
	fi
	if test -f "$g_dir_matnr/MARC.txt"
	then
	    rm -v "$g_dir_matnr/MARC.txt"
	fi
	if test -f "$g_dir_matnr/MVKE.txt"
	then
	    rm -v "$g_dir_matnr/MVKE.txt"
	fi
	f_rm_lock
    fi

    f_msg "E901: Cancelled, bye"

} # f_trap()

f_rm_lock()
{
    if test -f "$g_file_lock"
    then
	rm "$g_file_lock"
    fi
}

#
# f_cklock() -- if lock file deleted, exit
#
f_cklock()
{
    l_ckl_outfile="$1"

    if test ! -f "$g_file_lock"
    then
	if test "$l_ckl_outfile" != ""
	then
	    if test -f "$l_ckl_outfile"
	    then
		rm -v "$l_ckl_outfile"
	    fi
	fi
	f_msg "W100: lock file removed"
	f_msg "E101:     $g_file_lock"
    fi
}

#
# f_gen_file() -- create a specific MATNR File
#
f_gen_file()
{
    l_gf_template="$1"
    l_gf_dir_matnr="$2"
    l_gf_output="$3"
    l_gf_uf=""

    uname -a
    f_msg "I200: creating $l_gf_output"
    if test ! -d "$l_gf_dir_matnr"
    then
	f_msg "E201: missing dir $l_gf_dir_matnr"
    fi

    if test -f "$l_gf_output"
    then
	rm "$l_gf_output" > /dev/null 2>&1
    fi

    f_cklock
    for l_gf_uf in `find $l_gf_dir_matnr -type f -name "${l_gf_template}*.xz" | sort`
    do
	f_cklock "$g_file_lock"
	f_msg "I210: Processing $l_gf_uf"
	time unxz < "$l_gf_uf" | jcat -v -r 1500 -s 500 \
	     | grep '^|' | grep -v 'MATNR'             \
	     | sed 's/^|//;s/ *|/|/g;s/| */|/g' >> "$l_gf_output"
	if test "$?" -ne "0"
	then
	    f_msg "E211: Failure occurred on $l_gf_uf"
	fi
    done
    f_msg "I220: COMPLETE $l_gf_output"
    f_cklock

} # END: f_gen_file()

#
# f_create_matnr() -- create jmerge input from all MATNR Files
#
f_create_matnr()
{
    g_generating_files="Y"

    l_gc_dir_matnr="$WDIR/MATNR/in"
    l_gc_log_mara=$g_dir_matnr/results_MARA.txt
    l_gc_log_marc=$g_dir_matnr/results_MARC.txt
    l_gc_log_mvke=$g_dir_matnr/results_MVKE.txt

    if test ! -d "$l_gc_dir_matnr"
    then
	f_msg "E302: missing dir $l_gc_dir_matnr"
    fi
    if test ! -d "$l_gc_dir_matnr"
    then
	f_msg "E304: missing dir $l_gc_dir_matnr"
    fi

    f_msg "I310: START File Create"

    f_gen_file "MARA" "$l_gc_dir_matnr" "$g_dir_matnr/MARA.txt" > "$l_gc_log_mara" 2>&1 &
    g_gv_pid_mara="$!"
    f_gen_file "MARC" "$l_gc_dir_matnr" "$g_dir_matnr/MARC.txt" > "$l_gc_log_marc" 2>&1 &
    g_gv_pid_marc="$!"
    f_gen_file "MVKE" "$l_gc_dir_matnr" "$g_dir_matnr/MVKE.txt" > "$l_gc_log_mvke" 2>&1 &
    g_gv_pid_mvke="$!"

    f_msg "I390: WAITING for background jobs: $g_gv_pid_mara $g_gv_pid_marc $g_gv_pid_mvke"
    sleep 2 # seems some sh's wait is broken, so this
    wait $g_gv_pid_mara
    g_gv_pid_mara=""
    wait $g_gv_pid_marc
    g_gv_pid_marc=""
    wait $g_gv_pid_mvke
    g_gv_pid_mvke=""

    f_msg "I399: COMPLETE File Create"

    g_generating_files="N"
    if test -f "$g_file_lock"
    then
	rm "$g_file_lock"
    fi

} # END: f_create_matnr()

#
# f_jmerge() -- merge MATNR Files
f_jmerge()
{
    l_jm_sort="$1"
    l_jm_rmode="$2"
    l_jm_prog_jmerge="$HOME/src/util/jmerge"
    l_jm_file_key="$g_dir_merge/MARA.txt"
    l_jm_file_d01="$g_dir_merge/$l_jm_rmode.txt"
    l_jm_file_out="$g_dir_matnr/out_mara_$l_jm_rmode.txt"
    l_file_stat="$g_dir_matnr/stat_mara_$l_jm_rmode.txt"

    if test "$l_jm_sort" = "YES"
    then
	l_jm_arg_sort=""
    else
	l_jm_arg_sort="-u"
    fi

    if test ! -x "$l_jm_prog_jmerge"
    then
	f_msg "E401: Missing $l_jm_prog_jmerge"
    fi
    if test ! -r "$l_jm_file_key"
    then
	f_msg "E402: Missing $l_jm_file_key"
    fi
    if test ! -r "$l_jm_file_d01"
    then
	f_msg "E403: Missing $l_jm_file_d01"
    fi

    f_cklock

    if test -f "$l_jm_file_out"
    then
	rm  "$l_jm_file_out"
    fi
    if test -f "$l_file_stat"
    then
	rm  "$l_file_stat"
    fi

    uname -a | fmt
    echo ""
    f_msg "I410: START $l_jm_rmode"
    time $l_jm_prog_jmerge $l_jm_arg_sort -d 124 -vvv \
	     -o "$l_jm_file_out"                      \
             -i "$l_jm_file_key"                      \
             -K "$l_file_stat"                        \
             "$l_jm_file_d01"
    if test "$?" -ne "0"
    then
	f_msg "E411: FAILED jmerge processing $l_jm_rmode"
    fi
    f_msg "I412: END   $l_jm_rmode"

} # END: f_jmerge()

#
# main
#
g_sname="$0"
g_generating_files="N"
g_file_lock=$TMPDIR/locks/LOCK_MATNR_jmerge.txt
g_gv_pid_mara=""
g_gv_pid_marc=""
g_gv_pid_mvke=""

f_msg "I001: $g_sname START"
if test ! -d "$TMPDIR"
then
    f_msg "E002: missing dir $TMPDIR (TMPDIR)"
fi

case "$HOST" in
    "qball"|"fuzzball")
	g_dir_matnr="$TMPDIR/MATNR"
	g_dir_merge="$WDIR/jmerge"
	;;
    *)
	g_dir_matnr="$TMPDIR/MATNR"
	g_dir_merge="$WDIR/merge"
	;;
esac

if test ! -d "$g_dir_matnr"
then
    mkdir "$g_dir_matnr" && chmod 700 "$g_dir_matnr"
    if test ! -d "$g_dir_matnr"
    then
	f_msg "E003: missing dir $g_dir_matnr"
    fi
fi

trap f_trap INT TERM
if test -f "$g_file_lock"
then
    f_msg "W010: lock file found, leaving"
    f_msg "E011:     $g_file_lock"
fi
date > "$g_file_lock"

f_msg "I021: Creating MATNR Files"
f_create_matnr # create input files for use in jmerge
exit 0

f_msg "I021: generating Match for MARC"
f_jmerge "YES" "MARC" > "$g_dir_matnr/log_mara_MARC.txt" 2>&1
f_msg "I022: generating Match for MVKE"
f_jmerge "YES" "MVKE" > "$g_dir_matnr/log_mara_MVKE.txt" 2>&1

#--- DONE
f_msg "I090: $g_sname COMPLETE"
f_rm_lock
exit 0

